import { EdgesGeometry, Geometry, LineSegments, MeshBasicMaterial, Shape, Vector2 } from 'three';
import { polar } from './GeUtils';
import { edgeMaterial } from './Material';
import { SimpleBoard } from './SimpleBoard';
//解析二维圆弧类.
export class Arc2d
{
    m_StartAn: number;
    m_EndAn: number;
    m_StartPoint: Vector2;
    m_EndPoint: Vector2;
    m_Center: Vector2;
    m_Radius: number;
    constructor(p1: Vector2, p2: Vector2, bul: number)
    {
        this.m_StartPoint = p1.clone();
        this.m_EndPoint = p2.clone();

        let vec: Vector2 = p2.clone().sub(p1);
        let len = vec.length();
        let an = vec.angle();
        this.m_Radius = len / Math.sin(2 * Math.atan(bul)) / 2;
        let allAngle = Math.atan(bul) * 4;
        let delDis = bul * len / 2;
        let toDis = this.m_Radius - delDis;
        an += Math.PI * 0.5;

        this.m_Center = p1.clone().add(p2);
        this.m_Center.multiplyScalar(0.5);

        polar(this.m_Center, an, toDis);

        this.m_StartAn = p1.clone().sub(this.m_Center).angle();
        this.m_EndAn = p2.clone().sub(this.m_Center).angle();
        if (bul < 0)
        {
            //一个神奇的特性 它需要这么做
            this.m_StartAn -= Math.PI;
            this.m_EndAn -= Math.PI;
        }
    }
}

//创建轮廓 通过点表和凸度
export function createPath(pts: Vector2[], buls: number[], shapeOut?: Shape): Shape
{
    let shape = shapeOut || new Shape();
    if (pts.length === 0) return shape;
    let firstPt = pts[0];

    shape.moveTo(firstPt.x, firstPt.y);
    for (let i = 0; i < pts.length - 1; i++)
    {
        let nextPt = pts[i + 1];
        if (buls[i] == 0)
        {
            shape.lineTo(nextPt.x, nextPt.y);
        }
        else
        {
            let pt = pts[i];
            //参考
            //http://www.dorodnic.com/blog/tag/three-js/ 绘制一个齿轮
            //https://www.kirupa.com/html5/drawing_circles_canvas.htm //html5
            let arc2 = new Arc2d(pt, nextPt, buls[i]);
            let cen = arc2.m_Center;
            shape.absarc(cen.x, cen.y, arc2.m_Radius, arc2.m_StartAn, arc2.m_EndAn, buls[i] < 0);
        }
    }
    return shape;
}



//创建板件 暂时这么写


export function createTemplateBoard(brDataList: any[], material: MeshBasicMaterial)
{
    let meshs = [];
    let edgesa = [];

    let relations = {
        blockMeshMap: new Map<number, number>(),
        meshBlockMap: new Map<number, number>()
    };
    for (let d of brDataList)
    {
        let board = new SimpleBoard(d);
        let { mesh, edges } = board.createBoard(material);
        meshs.push(mesh);
        edgesa.push(...edges);

        if (board.DataID)
        {
            relations.blockMeshMap.set(board.DataID, mesh.id);
            relations.meshBlockMap.set(mesh.id, board.DataID);
        }
    }
    return { meshs, edgesa, relations };
}

export function createEdge(geo: Geometry): LineSegments
{
    let edge = new EdgesGeometry(geo, 1);
    return new LineSegments(edge, edgeMaterial);
}
